﻿Type=Class
Version=4.3
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
'	BeaconMonitor Class module

Private Sub Class_Globals
	'	this Type has 2 properties:
	'	EventName is the 'standard' EventName property used to prefix sub names
	'	Target is the module (Activity, Service etc) in which to raise events
	Type BeaconMonitorListener( _
		EventName As String, _
		Target As Object _
	)
	
	Private EVENT_BEACON_FOUND As String="BeaconFound"
	Private EVENT_ENTER_REGION As String="EnterRegion"
	Private EVENT_EXIT_REGION As String="ExitRegion"
	Private EVENT_SCAN_COMPLETE As String="ScanComplete"
	Private EVENT_SCAN_IDLE As String="ScanIdle"
	Private EVENT_SCAN_START As String="ScanStart"
	
	Private mAutoScanEnabled As Boolean
	Private mBeaconMonitorListeners As List
	Private mIBeaconProtocol As IBeaconProtocol
	Private mLoggingEnabled As Boolean
	Private mScanIdleTimer As Timer
End Sub

'	Add a listener.
'	BeaconMonitor will now raise it's events in the module set in the listener.
'	Multiple listeners (from different modules) can be added.
Public Sub AddListener(BeaconMonitorListener1 As BeaconMonitorListener)
	mBeaconMonitorListeners.Add(BeaconMonitorListener1)
	DebugLog("listener added")
End Sub

'	Configure the device's bluetooth adapter
'	ConfigureBluetoothAdapter will return False if unable to configure the bluetooth adapter.
'	Failure to configure the bluetooth adapter usually means the device's bluetooth is not enabled.
Public Sub ConfigureBluetoothAdapter As Boolean
	Return mIBeaconProtocol.ConfigureBluetoothAdapter
End Sub

Private Sub DebugLog(LogMessage As String)
	If mLoggingEnabled Then
		Log("BeaconMonitor: "&LogMessage)
	End If
End Sub

Public Sub GetIBeaconsByProximity As IBeacon()
	Return mIBeaconProtocol.GetIBeaconsByProximity
End Sub

'	Get the idle time between scans when auto scanning is enabled.
'	Units of milliseconds.
Public Sub GetScanIdleTime As Long
	Return mScanIdleTimer.Interval
End Sub

Private Sub IBeaconListener_BeaconFound(IBeacon1 As IBeacon)
	DebugLog(EVENT_BEACON_FOUND&" "&IBeacon1.ToString)
	RaiseEvent2(EVENT_BEACON_FOUND, IBeacon1)
End Sub

Private Sub IBeaconListener_EnterRegion(IBeacon1 As IBeacon)
	DebugLog(EVENT_ENTER_REGION&" "&IBeacon1.ToString)
	RaiseEvent2(EVENT_ENTER_REGION, IBeacon1)
End Sub

Private Sub IBeaconListener_ExitRegion(IBeacon1 As IBeacon)
	DebugLog(EVENT_EXIT_REGION&" "&IBeacon1.ToString)
	RaiseEvent2(EVENT_EXIT_REGION, IBeacon1)
End Sub

Private Sub IBeaconListener_OperationError(Status As Int)
	DebugLog("the OperationError event has been raised with a Status value of "&Status)
End Sub

Private Sub IBeaconListener_SearchState(State As Int)
	Select State
		Case mIBeaconProtocol.SEARCH_END_EMPTY, mIBeaconProtocol.SEARCH_END_SUCCESS
			Dim BeaconsFound As Boolean=(State=mIBeaconProtocol.SEARCH_END_SUCCESS)
			DebugLog(EVENT_SCAN_COMPLETE)
			RaiseEvent3(EVENT_SCAN_COMPLETE, BeaconsFound)
			If mAutoScanEnabled Then
				mScanIdleTimer.Enabled=True
				DebugLog(EVENT_SCAN_IDLE)
				RaiseEvent(EVENT_SCAN_IDLE)
			Else
				StopScan
			End If
		Case mIBeaconProtocol.SEARCH_STARTED
			DebugLog(EVENT_SCAN_START)
			RaiseEvent(EVENT_SCAN_START)
	End Select
End Sub

'	Initailize the BeaconMonitor
'	BeasonMonitor raises these events:
'		BeaconFound(IBeacon1 As IBeacon)
'		EnterRegion(IBeacon1 As IBeacon)
'		ExitRegion(IBeacon1 As IBeacon)
'		ScanComplete(BeaconsFound As Boolean)
'		ScanIdle
'		ScanStart
'	Events are raised in a module when a module adds a listener to the BeaconMonitor.
Public Sub Initialize
	'	auto scan disabled by default
	mAutoScanEnabled=False
	mBeaconMonitorListeners.Initialize
	mIBeaconProtocol.Initialize
	'	logging disabled by default
	mLoggingEnabled=False
	'	default scan idle time is 10 seconds
	mScanIdleTimer.Initialize("ScanIdle", 10*1000)
	
	Dim IBeaconListener1 As IBeaconListener
	IBeaconListener1.Initialize("IBeaconListener")
	mIBeaconProtocol.SetListener(IBeaconListener1)
End Sub

Public Sub IsAutoScanEnabled As Boolean
	Return mAutoScanEnabled
End Sub

Public Sub IsScanning As Boolean
	Return mIBeaconProtocol.IsScanning
End Sub

Private Sub RaiseEvent(EventName As String)
	Dim SubName As String
	For Each BeaconMonitorListener1 As BeaconMonitorListener In mBeaconMonitorListeners
		SubName=BeaconMonitorListener1.EventName&"_"&EventName
		If SubExists(BeaconMonitorListener1.Target, SubName) Then
			CallSubDelayed(BeaconMonitorListener1.Target, SubName)
		Else
			DebugLog("callback sub '"&EventName&"' not found")
		End If
	Next
End Sub

Private Sub RaiseEvent2(EventName As String, IBeacon1 As IBeacon)
	Dim SubName As String
	For Each BeaconMonitorListener1 As BeaconMonitorListener In mBeaconMonitorListeners
		SubName=BeaconMonitorListener1.EventName&"_"&EventName
		If SubExists(BeaconMonitorListener1.Target, SubName) Then
			CallSubDelayed2(BeaconMonitorListener1.Target, SubName, IBeacon1)
		Else
			DebugLog("callback sub '"&EventName&"' not found")
		End If
	Next
End Sub

Private Sub RaiseEvent3(EventName As String, BooleanValue As Boolean)
	Dim SubName As String
	For Each BeaconMonitorListener1 As BeaconMonitorListener In mBeaconMonitorListeners
		SubName=BeaconMonitorListener1.EventName&"_"&EventName
		If SubExists(BeaconMonitorListener1.Target, SubName) Then
			CallSubDelayed2(BeaconMonitorListener1.Target, SubName, BooleanValue)
		Else
			DebugLog("callback sub '"&EventName&"' not found")
		End If
	Next
End Sub

'	Remove a listener.
'	BeaconMonitor will no longer raise events for the removed listener's module.
Public Sub RemoveListener(BeaconMonitorListener1 As BeaconMonitorListener)
	Dim Index As Int=mBeaconMonitorListeners.IndexOf(BeaconMonitorListener1)
	If Index<>-1 Then
		mBeaconMonitorListeners.RemoveAt(Index)
		DebugLog("listener removed")
	Else
		DebugLog("failed to remove listener - listener not found")
	End If
End Sub

'	Reset tells BeaconMonitor to forget about any previously found nearby beacon.
Public Sub Reset
	mIBeaconProtocol.Reset
End Sub

Private Sub ScanIdle_Tick
	DebugLog("ScanIdle_Tick")
	mScanIdleTimer.Enabled=False
	If mAutoScanEnabled Then
		StartScan
	End If
End Sub

Public Sub SetAutoScanEnabled(AutoScanEnabled As Boolean)
	mAutoScanEnabled=AutoScanEnabled
End Sub

'	Enable or disable logging within the BeaconMonitor.
Public Sub SetLoggingEnabled(LoggingEnabled As Boolean)
	mLoggingEnabled=LoggingEnabled
End Sub

'	Set the idle time between scans when auto scanning is enabled.
'	Units of milliseconds.
Public Sub SetScanIdleTime(IdleTime As Long)
	mScanIdleTimer.Interval=IdleTime
End Sub

'	Set the UUID of beacons to search for.
'	Pass an empty string to clear any existing filter and search for all beacons.
Public Sub SetUUIDFilter(Uuid As String)
	If Uuid="" Then
		mIBeaconProtocol.SetScanUUID(Null)
	Else
		Dim ByteConverter1 As ByteConverter
		mIBeaconProtocol.SetScanUUID(ByteConverter1.HexToBytes(Uuid))
	End If
End Sub

Public Sub StartScan
	mIBeaconProtocol.StartScan
End Sub

Public Sub StopScan
	If mScanIdleTimer.Enabled Then
		mScanIdleTimer.Enabled=False
	End If
	If mIBeaconProtocol.IsScanning Then
		mIBeaconProtocol.StopScan
	End If
End Sub
